@@ -59,7 +59,8 @@ cog <- function(val = NULL, desc = "", group = "common",
     # geo         = as.cogGeo,
     # rel         = as.cogRel,
     # hier        = as.cogHier,
-    href          = as.character
+    href          = as.character,
+    href_hash     = as.character
   )
 
   types <- names(cog_types)
@@ -134,19 +135,22 @@ infer_cog_type <- function(val) {
 #' @param val A string indicating the value of the filter.
 #' @param desc a description for this cognostic value
 #' @param group optional categorization of the cognostic for organizational purposes in the viewer (currently not implemented in the viewer)
+#' @param type of either "href" or "href_hash". "href" will open the link in a new page. "href_hash" will update the page's hash and reload the page (useful when changing state inside an iframe)
 #' @param default_label should this cognostic be used as a panel label in the viewer by default?
 #' @param default_active should this cognostic be active (available for sort / filter / sample) by default?
 #' @param filterable should this cognostic be filterable?  Default is \code{TRUE}.  It can be useful to set this to \code{FALSE} if the cognostic is categorical with many unique values and is only desired to be used as a panel label.
 #' @param sortable should this cognostic be sortable?
 #' @export
 cog_disp_filter <- function(display, var, val,
-  desc = "link", group = "common",
+  desc = "link", group = "common", type = c("href", "href_hash"),
   default_label = FALSE, default_active = FALSE,
   filterable = FALSE, sortable = FALSE) {
   x <- paste0("#display=", display, "&filter=var:",
     var, ";type:select;val:", val)
 
-  cog(x, type = "href", desc = desc, group = group, 
+  type <- match.arg(type)
+
+  cog(x, type = type, desc = desc, group = group, 
     default_label = default_label,
     default_active = default_active,
     filterable = filterable, sortable = sortable,
@@ -158,6 +162,7 @@ cog_disp_filter <- function(display, var, val,
 #' Create href to be used as cognostics in a trelliscope display
 #'
 #' @param x URL to link to
+#' @param type of either "href" or "href_hash". "href" will open the link in a new page. "href_hash" will update the page's hash and reload the page (useful when changing state inside an iframe)
 #' @param desc,group,default_label,default_active,filterable,sortable,log arguments passed to \code{\link{cog}}
 #'
 #' @seealso \code{\link{cog}}
@@ -180,11 +185,13 @@ cog_disp_filter <- function(display, var, val,
 #'   trelliscope(name = "iris_species", ncol = 3)
 #' }
 #' @export
-cog_href <- function(x, desc = "link", group = "common",
+cog_href <- function(x, desc = "link", group = "common", type = c("href", "href_hash"),
   default_label = FALSE, default_active = FALSE, filterable = FALSE,
   sortable = FALSE, log = FALSE) {
 
-  cog(x, type = "href", desc = desc, group = group, default_label = default_label,
+  type <- match.arg(type)
+
+  cog(x, type = type, desc = desc, group = group, default_label = default_label,
     default_active = default_active, filterable = filterable, sortable = sortable,
     log = log)
 }
@@ -305,7 +312,7 @@ bind_cog_list_and_descs <- function(cog_list) {
 
 #' @importFrom autocogs panel_cogs
 cog_df_info <- function(x, panel_col, state, auto_cog = FALSE, nested_data_list = NULL,
-  nested_cog_attrs = NULL) {
+  nested_cog_attrs = NULL, cond_cols = NULL) {
 
   atomic_cols <- names(x)[sapply(x, is.atomic)]
   non_atomic_cols <- setdiff(names(x), c(atomic_cols, panel_col))
@@ -315,7 +322,14 @@ cog_df_info <- function(x, panel_col, state, auto_cog = FALSE, nested_data_list
     stop_nice("There must be at least one atomic column in the data frame passed in",
       "to trelliscope.data.frame")
 
-  cond_cols <- find_cond_cols(x[atomic_cols], is_nested)
+  if (is.null(cond_cols)) {
+    cond_cols <- find_cond_cols(x[atomic_cols], is_nested)
+  } else {
+    if (any(sapply(x[cond_cols], function(a) any(is.na(a)))))
+      stop("cond_cols must not have any NA values")
+    if (length(unique(do.call(paste, c(x[cond_cols], sep = "_")))) != nrow(x))
+      stop("cond_cols must uniquely define each row")
+  }
 
   # if we are no longer sorted by a cond_col but are sorted by something else
   # and if sort state is not already specified, then set that as state
@@ -468,3 +482,61 @@ cog_df_info <- function(x, panel_col, state, auto_cog = FALSE, nested_data_list
     state = state
   )
 }
+
+#' Create a cognostics template that can be edited and used to specify
+#' cognostics in a display
+#' @param x a data frame that will be used as an input for the
+#' trelliscope display. If NULL, a blank template will be created.
+#'
+#' @note The input `x` can be a starting point and does not need to
+#' contain all variables that may be added later.
+#' Also note that after you edit the output (for example using the
+#' editData package, or writing to a csv and editing in Excel) to reflect
+#' how you would like the cognostics to display in the viewer, you can
+#' Add this specification to your trelliscope display with
+#' [add_cog_template()].
+#' @export
+create_cog_template <- function(x = NULL) {
+  if (is.null(x))
+    x <- data.frame(example_variable = 1:10)
+  csv <- dplyr::bind_rows(lapply(names(x), function(nm) {
+    cg <- attributes(cog(x[[nm]]))$cog_attrs
+    dplyr::tibble(
+      name = nm,
+      desc = cg$desc,
+      type = cg$type,
+      group = cg$group,
+      defLabel = cg$defLabel,
+      defActive = cg$defActive,
+      filterable = cg$filterable,
+      log = cg$log
+    )
+  }))
+
+  message(
+    "A data frame with a cognostics specification template has been ",
+    " created.\n",
+    "You can edit this data frame (either write to csv and edit, or use ",
+    "the editData package) and provide it to a trelliscope display ",
+    "definition by adding %>% add_cog_template(updated_df) to your ",
+    "trellscope display specification.")
+  message("The following columns are present in the template:")
+  message("  name: name of the cognostic (must match a variable name in ",
+    "your dataset)")
+  message("  desc: description of the cognostic")
+  message("  type: type of the cognostic")
+  message("  group: group(s) to organize the congostic into in the UI. ",
+    "If more than one group separate them with a semicolon.")
+  message("  defLabel: should this cognostic be used as a panel label in ",
+    "the viewer by default?")
+  message("  defActive: should this cognostic be active (available for ",
+    "sort / filter / sample) by default?")
+  message("  filterable: should this cognostic be filterable? Default is ",
+    "TRUE. It can be useful to set this to FALSE if the cognostic is ",
+    "categorical with many unique values and is only desired to be used ",
+    "as a panel label.")
+  message(" log: when being used in the viewer for visual univariate ",
+    "and bivariate filters, should the variable be log transformed? ")
+
+  csv
+}
